<?PHP
/* $Id: media_actions.inc,v 1.1 2007/11/14 21:29:49 pfournier Exp $ */

/**
 * @file
 * Contains actions for use with the actions module.
 * These actions manage video file storage on bliptv.
 */

/**
 * Action definition: put file on bliptv.
 *
 */
function action_bliptv_store($op, $edit = array(), &$node) {
  switch ($op) {
    case 'do':
      if (_bliptv_send($node, $edit)) {
        if (!$edit['defer']) {
          node_save($node);
        }
        watchdog('action', t('Sent node id %id media to blip.tv', array('%id' => intval($node->nid))));
      }
      break;

    case 'metadata':
      return array(
        'description' => t('Send media to blip.tv'),
        'type' => t('Media'),
        'batchable' => TRUE,
        'configurable' => TRUE,
      );

    // Return an HTML config form for the action.
    case 'form':
      // default values for form
      if (!isset($edit['bliptv_username'])) $edit['bliptv_username'] = '';
      if (!isset($edit['bliptv_password'])) $edit['bliptv_password'] = '';
      if (!isset($edit['bliptv_timeout'])) $edit['bliptv_timeout'] = '300';
      if (!isset($edit['bliptv_license'])) $edit['bliptv_license'] = '1';
      if (!isset($edit['blitv_signature'])) $edit['blitv_signature'] = '';
      if (!isset($edit['bliptv_srcfield'])) $edit['bliptv_srcfield'] = 'local_video';
      if (!isset($edit['bliptv_dstfield'])) $edit['bliptv_dstfield'] = 'remote_video';
      $form = array();

      // add form components
      $form['bliptv_username'] = array(
        '#type' => 'textfield',
        '#title' => t('blip.tv user name'),
        '#description' => t("Name of the blip.tv user."),
        '#default_value' => $edit['bliptv_username'],
        '#size' => '20',
        '#maxlength' => '254',
      );
      $form['bliptv_password'] = array(
        '#type' => 'textfield',
        '#title' => t('blip.tv password'),
        '#default_value' => $edit['bliptv_password'],
        '#size' => '20',
        '#maxlength' => '254',
        '#description' => t('Password for the blip.tv user above.'),
      );
      $form['bliptv_timeout'] = array(
        '#type' => 'textfield',
        '#title' => t('Upload timeout'),
        '#default_value' => $edit['bliptv_timeout'],
        '#after_field' => t('seconds'),
        '#size' => '4',
        '#maxlength' => '4',
        '#description' => t('How long we should wait for the transfer from our server to blip.tv server to complete.'),
      );
      $license['-1'] = 'No license (All rights reserved)';
      $license['1'] = 'Creative Commons Attribution';
      $license['2'] = 'Creative Commons Attribution-NoDerivs';
      $license['3'] = 'Creative Commons Attribution-NonCommercial-NoDerivs';
      $license['4'] = 'Creative Commons Attribution-NonCommercial';
      $license['5'] = 'Creative Commons Attribution-NonCommercial-ShareAlike';
      $license['6'] = 'Creative Commons Attribution-ShareAlike';
      $license['7'] = 'Public Domain';
      $form['bliptv_license'] =  array(
        '#type' => 'select',
        '#title' => t('Global License Type'),
        '#options' => $license,
        '#default_value' => $edit['bliptv_license'],
        '#description' => t('All video will be uploaded and attached with the specified license. Description of licenses here <a href="http://creativecommons.org/licenses/">www.creativecommons.org/licenses</a>'),
      );
      $form['bliptv_signature'] = array(
        '#type' => 'textfield',
        '#title' => t('Global Signature'),
        '#length' => 5,
        '#default_value' => $edit['bliptv_signature'],
        '#description' => t('This text will be appended to any video description uploaded to your blip.tv account e.g. a link back to your site from blip.tv')
      );
      $form['bliptv_srcfield'] = array(
        '#type' => 'textfield',
        '#title' => t('Local Video Field Name'),
        '#default_value' => $edit['bliptv_srcfield'],
        '#description' => t("Name of the node field containing the video file path."),
      );
      $form['bliptv_dstfield'] = array(
        '#type' => 'textfield',
        '#title' => t('Remote Video Field Name'),
        '#default_value' => $edit['bliptv_dstfield'],
        '#description' => t("Name of the node field where the uploaded video URL will be stored."),
      );
      
      return $form;
      
    // Validate the HTML form.
    case 'validate':
      $errors = array();
      if ($edit['bliptv_username'] == "") {
        $errors['bliptv_username'] =  t("You must provide a blip.tv user name.");
      }
      if ($edit['bliptv_password'] == "") {
        $errors['bliptv_password'] =  t("You must provide a blip.tv password.");
      }
      if (!is_numeric($edit['bliptv_timeout']) || ((int)$edit['bliptv_timeout'] < 0)) {
        $errors['bliptv_timeout'] = t("The timeout must be a non-negative numeric value.");
      }
      
      foreach ($errors as $name => $message) {
        form_set_error($name, $message);
      }
      
      return count($errors) == 0;

    // Process the HTML form to store configuration.
    case 'submit':
      $params = array(
        'bliptv_username' => $edit['bliptv_username'],
        'bliptv_password' => $edit['bliptv_password'],
        'bliptv_timeout'  => $edit['bliptv_timeout'],
        'bliptv_license'  => $edit['bliptv_license'],
        'blitv_signature' => $edit['blitv_signature'],
        'bliptv_srcfield' => $edit['bliptv_srcfield'],
        'bliptv_dstfield' => $edit['bliptv_dstfield'],
      );
      return $params;
  }
}

/**
 * Action definition:update file on bliptv.
 *
 */
function action_bliptv_update($op, $edit = array(), &$node) {
  switch ($op) {
    case 'do':
      if (_bliptv_update($node, $edit)) {
        if (!$edit['defer']) {
          node_save($node);
        }
        watchdog('action', t('Updated node id %id media on blip.tv', array('%id' => intval($node->nid))));
      }
      break;

    case 'metadata':
      return array(
        'description' => t('Update media on blip.tv'),
        'type' => t('Media'),
        'batchable' => TRUE,
        'configurable' => TRUE,
      );

    // Return an HTML config form for the action.
    case 'form':
      // default values for form
      if (!isset($edit['bliptv_username'])) $edit['bliptv_username'] = '';
      if (!isset($edit['bliptv_password'])) $edit['bliptv_password'] = '';
      if (!isset($edit['bliptv_timeout'])) $edit['bliptv_timeout'] = '300';
      if (!isset($edit['bliptv_license'])) $edit['bliptv_license'] = '1';
      if (!isset($edit['blitv_signature'])) $edit['blitv_signature'] = '';
      if (!isset($edit['bliptv_srcfield'])) $edit['bliptv_srcfield'] = 'local_video';
      if (!isset($edit['bliptv_dstfield'])) $edit['bliptv_dstfield'] = 'remote_video';
      $form = array();

      // add form components
      $form['bliptv_username'] = array(
        '#type' => 'textfield',
        '#title' => t('blip.tv user name'),
        '#description' => t("Name of the blip.tv user."),
        '#default_value' => $edit['bliptv_username'],
        '#size' => '20',
        '#maxlength' => '254',
      );
      $form['bliptv_password'] = array(
        '#type' => 'textfield',
        '#title' => t('blip.tv password'),
        '#default_value' => $edit['bliptv_password'],
        '#size' => '20',
        '#maxlength' => '254',
        '#description' => t('Password for the blip.tv user above.'),
      );
      $form['bliptv_timeout'] = array(
        '#type' => 'textfield',
        '#title' => t('Upload timeout'),
        '#default_value' => $edit['bliptv_timeout'],
        '#after_field' => t('seconds'),
        '#size' => '4',
        '#maxlength' => '4',
        '#description' => t('How long we should wait for the transfer from our server to blip.tv server to complete.'),
      );
      $license['-1'] = 'No license (All rights reserved)';
      $license['1'] = 'Creative Commons Attribution';
      $license['2'] = 'Creative Commons Attribution-NoDerivs';
      $license['3'] = 'Creative Commons Attribution-NonCommercial-NoDerivs';
      $license['4'] = 'Creative Commons Attribution-NonCommercial';
      $license['5'] = 'Creative Commons Attribution-NonCommercial-ShareAlike';
      $license['6'] = 'Creative Commons Attribution-ShareAlike';
      $license['7'] = 'Public Domain';
      $form['bliptv_license'] =  array(
        '#type' => 'select',
        '#title' => t('Global License Type'),
        '#options' => $license,
        '#default_value' => $edit['bliptv_license'],
        '#description' => t('All video will be uploaded and attached with the specified license. Description of licenses here <a href="http://creativecommons.org/licenses/">www.creativecommons.org/licenses</a>'),
      );
      $form['bliptv_signature'] = array(
        '#type' => 'textfield',
        '#title' => t('Global Signature'),
        '#length' => 5,
        '#default_value' => $edit['bliptv_signature'],
        '#description' => t('This text will be appended to any video description uploaded to your blip.tv account e.g. a link back to your site from blip.tv')
      );
      $form['bliptv_srcfield'] = array(
        '#type' => 'textfield',
        '#title' => t('Local Video Field Name'),
        '#default_value' => $edit['bliptv_srcfield'],
        '#description' => t("Name of the field containing the video file path."),
      );
      $form['bliptv_dstfield'] = array(
        '#type' => 'textfield',
        '#title' => t('Remote Video Field Name'),
        '#default_value' => $edit['bliptv_dstfield'],
        '#description' => t("Name of the field where the uploaded video URL will be stored."),
      );
      
      return $form;
      
    // Validate the HTML form.
    case 'validate':
      $errors = array();
      if ($edit['bliptv_username'] == "") {
        $errors['bliptv_username'] =  t("You must provide a blip.tv user name.");
      }
      if ($edit['bliptv_password'] == "") {
        $errors['bliptv_password'] =  t("You must provide a blip.tv password.");
      }
      if (!is_numeric($edit['bliptv_timeout']) || ((int)$edit['bliptv_timeout'] < 0)) {
        $errors['bliptv_timeout'] = t("The timeout must be a non-negative numeric value.");
      }
      
      foreach ($errors as $name => $message) {
        form_set_error($name, $message);
      }
      
      return count($errors) == 0;
      
    // Process the HTML form to store configuration.
    case 'submit':
      $params = array(
        'bliptv_username' => $edit['bliptv_username'],
        'bliptv_password' => $edit['bliptv_password'],
        'bliptv_timeout'  => $edit['bliptv_timeout'],
        'bliptv_license'  => $edit['bliptv_license'],
        'blitv_signature' => $edit['blitv_signature'],
        'bliptv_srcfield' => $edit['bliptv_srcfield'],
        'bliptv_dstfield' => $edit['bliptv_dstfield'],
      );
      return $params;
  }
}

/**
 * Action definition: remove file from bliptv.
 *
 */
function action_bliptv_delete($op, $edit = array(), &$node) {
  switch ($op) {
    case 'do':
      if (_bliptv_delete($node, $edit)) {
        watchdog('action', t('Sent node id %id deleted from blip.tv', array('%id' => intval($node->nid))));
      }
      break;

    case 'metadata':
      return array(
        'description' => t('Remove media from blip.tv'),
        'type' => t('Media'),
        'batchable' => TRUE,
        'configurable' => TRUE,
      );

    // Return an HTML config form for the action.
    case 'form':
      // default values for form
      if (!isset($edit['bliptv_username'])) $edit['bliptv_username'] = '';
      if (!isset($edit['bliptv_password'])) $edit['bliptv_password'] = '';
      if (!isset($edit['bliptv_dstfield'])) $edit['bliptv_dstfield'] = 'remote_video';
      $form = array();

      // Add form components.
      $form['bliptv_username'] = array(
        '#type' => 'textfield',
        '#title' => t('blip.tv user name'),
        '#description' => t("Name of the blip.tv user."),
        '#default_value' => $edit['bliptv_username'],
        '#size' => '20',
        '#maxlength' => '254',
      );
      $form['bliptv_password'] = array(
        '#type' => 'textfield',
        '#title' => t('blip.tv password'),
        '#default_value' => $edit['bliptv_password'],
        '#size' => '20',
        '#maxlength' => '254',
        '#description' => t('Password for the blip.tv user above.'),
      );
      $form['bliptv_dstfield'] = array(
        '#type' => 'textfield',
        '#title' => t('Remote Video Field Name'),
        '#default_value' => $edit['bliptv_dstfield'],
        '#description' => t("Name of the field where the uploaded video URL will be stored."),
      );
      
      return $form;
      
    // Validate the HTML form.
    case 'validate':
      $errors = array();
      if ($edit['bliptv_username'] == "") {
        $errors['bliptv_username'] =  t("You must provide a blip.tv user name.");
      }
      if ($edit['bliptv_password'] == "") {
        $errors['bliptv_password'] =  t("You must provide a blip.tv password.");
      }
      
      foreach ($errors as $name => $message) {
        form_set_error($name, $message);
      }
      
      return count($errors) == 0;
      
    // Process the HTML form to store configuration.
    case 'submit':
      $params = array(
        'bliptv_username' => $edit['bliptv_username'],
        'bliptv_password' => $edit['bliptv_password'],
        'bliptv_dstfield' => $edit['bliptv_dstfield'],
      );
      return $params;
  }
}


/**
 * Sends file to bliptv.
 * @node Node containing file to send.
 * @params Action configuration parameters.
 * @blid_file_id If this is set, we will update the bliptv file instead of creating a new one.
 * @return False on error, TRUE on success.
 *
 */
function _bliptv_send($node, $params = array(), $blip_file_id = null) {
  // check user name
  if (! $username = $params['bliptv_username']) {
    $message = t("blip.tv username not set");
    watchdog('media_actions', $message, 'WATCHDOG_ERROR');
    return FALSE;
  }
  
  // check password
  if (! $password = $params['bliptv_password']) {
    $message = t("blip.tv password not set");
    watchdog('media_actions', $message, 'WATCHDOG_ERROR');
    return FALSE;
  }

  // check srcfield
  if ((! $local_video_field_name = $params['bliptv_srcfield'])) {
    $message = t("Local video field not set");
    watchdog('media_actions', $message, 'WATCHDOG_ERROR');
    return FALSE;
  }
  if (!array_key_exists($local_video_field_name, get_object_vars($node))) {
    $message = t("Local video field is not defined on node");
    watchdog('media_actions', $message, 'WATCHDOG_ERROR');
    return FALSE;
  }

  // check dstfield
  if ((! $remote_video_field_name = $params['bliptv_dstfield'])) {
    $message = t("Remote video field not set");
    watchdog('media_actions', $message, 'WATCHDOG_ERROR');
    return FALSE;
  }
  if (!array_key_exists($remote_video_field_name, get_object_vars($node))) {
    $message = t("Remote video field is not defined on node");
    watchdog('media_actions', $message, 'WATCHDOG_ERROR');
    return FALSE;
  }
  
  //@FIXME: Handle case where there is no video/video was deleted.
  $local_video_field = &$node->$local_video_field_name;
  $count = count($local_video_field);
  $file_path = $local_video_field[$count - 1]['filepath'];
  $file_name = basename($file_path);
    
  // Upload file to blip.tv
  $post_data['file'] = "@" . $_SERVER["DOCUMENT_ROOT"] . "/" . $file_path;
  $post_data['description'] = $node->body . $edit['bliptv_signature']; // Any valid text or HTML
  $post_data['title'] = $node->title; // Maximum length 255. No HTML.
  $post_data['post'] = 1; // Required: must set this to 1
  $post_data['license'] = $edit['bliptv_license']; // Defaults to "-1" or "No license"
  $post_data['categories_id'] = -1; // "-1" means no category. Never use a value of "0".
  $post_data['userlogin'] = $username; // valid username
  $post_data['password'] = $password; // valid password
  if (isset($blip_file_id)) {
      $post_data['id'] = $blip_file_id; // update the file
  }
  
  $response = array();

  $timeout = intval($edit['bliptv_timeout']);
  set_time_limit($timeout);
  
  // @TODO use drupal_http_request()
  // for encoding data, see http://api-drupal.pajunas.com/api/function/_openid_encode_message/DRUPAL-5
  // and also: http://www.w3.org/TR/html4/interact/forms.html
  $curl_session = curl_init();
  curl_setopt($curl_session, CURLOPT_URL, "http://www.blip.tv/?section=file&cmd=post&skin=api");
  
  curl_setopt($curl_session, CURLOPT_POST, 1 );
  curl_setopt($curl_session, CURLOPT_POSTFIELDS, $post_data);
  curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, TRUE);
  curl_setopt($curl_session, CURLOPT_TIMEOUT, $timeout);
  $response['data'] = curl_exec($curl_session);
  
  if (curl_errno($curl_session)) {
    $response['error'] = curl_error($curl_session); 
  }
  curl_close($curl_session);
  
  if ($response['error']) {
    drupal_set_message(" ".$response['error'], 'error');    
    return FALSE; 
  }
  
  $data = trim($response['data']);
  $vals = $index = $array = array();
  $parser = xml_parser_create();
  xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
  xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
  $xml_parse_result = xml_parse_into_struct($parser, $data, $vals, $index);

  if ($xml_parse_result == 0) {
    $file_id = 0;
    $file_url = "";
    drupal_set_message(t("Error in blip.tv answer: ") . xml_error_string(xml_get_error_code($parser)) . t(" at line ") . xml_get_current_line_number($parser), 'error');
  }
  else
  {
    $file_id = $vals[$index['item_id'][0]]['value'];
    $file_url = $vals[$index['link'][0]]['attributes']['href'];
  }
  
  xml_parser_free($parser);
  
  $remote_video_field = &$node->$remote_video_field_name;
  if ($file_id != $remote_video_field[0]['value']) {
    $remote_video_field[0]['embed'] = $file_url;
    $remote_video_field[0]['value'] = $file_id;
    $remote_video_field[0]['provider'] = 'bliptv';
    node_validate($node);
  }
    
  $local_video_field[$count - 1]['flags']['delete'] = TRUE;
  
  return TRUE;
}


/**
 * Update file on bliptv.
 * @node Node containing file to update.
 * @params Action configuration parameters.
 * @return FALSE on error, TRUE on success.
 *
 */
function _bliptv_update($node, $params = array()) {
  if ((! $remote_video_field_name = $params['bliptv_dstfield'])) {
    $message = t("Remote video field not set");
    watchdog('media_actions', $message, 'WATCHDOG_ERROR');
    return FALSE;
  }
  else {
    $remote_video_field = &$node->$remote_video_field_name;
    $blip_id = $remote_video_field[0]['value'];
    
    _bliptv_send($node, $params, $blip_id);
  }
}

/**
 * Delete file from bliptv.
 * @node Node containing file to delete.
 * @params Action configuration parameters.
 * @return FALSE on error, TRUE on success.
 *
 */
function _bliptv_delete($node, $params = array()) {
  // check user name
  if (! $username = $params['bliptv_username']) {
    $message = t("blip.tv username not set");
    watchdog('media_actions', $message, 'WATCHDOG_ERROR');
    return FALSE;
  }
  
  // check password
  if (! $password = $params['bliptv_password']) {
    $message = t("blip.tv password not set");
    watchdog('media_actions', $message, 'WATCHDOG_ERROR');
    return FALSE;
  }

  // check dstfield
  if ((! $remote_video_field_name = $params['bliptv_dstfield'])) {
    $message = t("Remote video field not set");
    watchdog('media_actions', $message, 'WATCHDOG_ERROR');
    return FALSE;
  }
  if (!array_key_exists($remote_video_field_name, get_object_vars($node))) {
    $message = t("Remote video field is not defined on node");
    watchdog('media_actions', $message, 'WATCHDOG_ERROR');
    return FALSE;
  }
  
  $remote_video_field = $node->$remote_video_field_name;  
  $bliptv_file_id = $remote_video_field[0]['value'];
    
  $reason = urlencode(t("Source node deleted"));
  $username = urlencode($username);
  $password = urlencode($password);
  $response = drupal_http_request("http://www.blip.tv/?section=file&cmd=delete&id=$bliptv_file_id&reason=$reason&userlogin=$username&password=$password&skin=api");
  
  if (isset($response->error)) {
    drupal_set_message($response->error);    
    return FALSE; 
  }
  
  $data = trim($response->data);
  $vals = $index = $array = array();
  $parser = xml_parser_create();
  xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
  xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
  $xml_parse_result = xml_parse_into_struct($parser, $data, $vals, $index);

  if ($xml_parse_result == 0) {
    drupal_set_message(t("Error in blip.tv answer: ") . xml_error_string(xml_get_error_code($parser)) . t(" at line ") . xml_get_current_line_number($parser), 'error');
  }
  else
  {
    if (!$vals[$index['deleted'][0]]['value']){
      drupal_set_message(t("Could not delete blip.tv file"), 'error');
    }
  }
  
  xml_parser_free($parser);
  
  $remote_video_field[0]['embed'] = '';
  $remote_video_field[0]['value'] = '';
  $remote_video_field[0]['provider'] = '';
  node_validate($node);
  
  return TRUE;
}